package
{
	import de.spring.sensors.SpringStreams;
	import de.spring.sensors.osmf.SpringStreamsOsmfPluginInfo;
	
	import flash.display.MovieClip;
	import flash.display.Sprite;
	import flash.events.MouseEvent;
	
	import org.osmf.containers.MediaContainer;
	import org.osmf.elements.ImageElement;
	import org.osmf.events.DisplayObjectEvent;
	import org.osmf.events.MediaFactoryEvent;
	import org.osmf.events.TimeEvent;
	import org.osmf.layout.LayoutMetadata;
	import org.osmf.layout.ScaleMode;
	import org.osmf.media.DefaultMediaFactory;
	import org.osmf.media.MediaElement;
	import org.osmf.media.MediaFactory;
	import org.osmf.media.MediaPlayer;
	import org.osmf.media.MediaPlayerSprite;
	import org.osmf.media.PluginInfoResource;
	import org.osmf.media.URLResource;
	import org.osmf.traits.LoadTrait;
	import org.osmf.traits.MediaTraitType;
	
	
	[SWF(width="800", height="600")]

	public class OsmfPlayer2 extends MovieClip
	{
		
		public static const STREAMING_MP4_PATH:String = "rtmp://cp67126.edgefcs.net/ondemand/mp4:mediapm/ovp/content/demo/video/elephants_dream/elephants_dream_768x428_24.0fps_408kbps.mp4";

		public var factory:MediaFactory;
		public var player:MediaPlayer;
		public var container:MediaContainer;
		public var mediaFactory:DefaultMediaFactory;
		
		
		//control bar declarations
		public var controlBar:Sprite;
		public var controlBG:ControlBG;
		public var pauseButton:PauseButton;
		public var playButton:PlayButton;
		public var progressBG:ProgressBG;
		public var progressCurrent:ProgressCurrent;

		/** if set to <code>true</code> a adapter will be loaded which implements 
		 * the SpringStreams sensor. if set to <code>false</code> the player will
		 * used directly via an implemented MediaPlayerStreamAdapter
		 */
		public var springUsePlugin:Boolean = true;
		private var sensor:de.spring.sensors.SpringStreams;
		
		
		public function OsmfPlayer2()
		{
			// Create the DefaultMediaFactory and add listeners 
			// for the loading of the plug-in. 
			
			trace("starting player app");
			
			factory = new DefaultMediaFactory(); 
			factory.addEventListener(MediaFactoryEvent.PLUGIN_LOAD, onPluginLoad); 
			factory.addEventListener(MediaFactoryEvent.PLUGIN_LOAD_ERROR, onPluginLoadError);

			if(springUsePlugin) {
				factory.loadPlugin(new PluginInfoResource(new SpringStreamsOsmfPluginInfo("test")));
			} else {
				sensor = new de.spring.sensors.SpringStreams("test");
				sensor.debug(true);
			}
			
			initPlayer();
			initControlBar();
		}
		
		
		protected function initPlayer():void
		{
			//the pointer to the media
			var resource:URLResource = new URLResource( STREAMING_MP4_PATH );
			
			var atts:Object = new Object();
			atts.stream = "flash/teststream";
			
			if(springUsePlugin) {
				resource.addMetadataValue(SpringStreamsOsmfPluginInfo.STREAM, atts.stream);
				resource.addMetadataValue(SpringStreamsOsmfPluginInfo.DEBUG, "true");
			}

			
			//creates and sets the MediaElement (generic) with a resource and path
			var element:MediaElement = factory.createMediaElement( resource );
			
			//the simplified api controller for media
			player = new MediaPlayer( element );

			if(!springUsePlugin) {
				sensor.track(new MediaPlayerStreamAdapter(player), atts);
			}			
			
			
			//Marker 5: Add MediaPlayer listeners for media size and current time change
			player.addEventListener( DisplayObjectEvent.MEDIA_SIZE_CHANGE, _onSizeChange );
			player.addEventListener( TimeEvent.CURRENT_TIME_CHANGE, _onProgress );
			
			//the container (sprite) for managing display and layout
			container = new MediaContainer();
			container.addMediaElement( element );
			
			//Marker 1: Add Bug Overlay
			var bugUrlResource:URLResource = new URLResource( "assets/osmf_stacked.png" );
			var bug:ImageElement = new ImageElement( bugUrlResource );
			var layoutData:LayoutMetadata = new LayoutMetadata();
			layoutData.right = 10;
			layoutData.bottom = 10;
			layoutData.scaleMode = ScaleMode.NONE;
			bug.metadata.addValue( LayoutMetadata.LAYOUT_NAMESPACE, layoutData );
			var bugLoadTrait:LoadTrait = bug.getTrait( MediaTraitType.LOAD ) as LoadTrait; 
			bugLoadTrait.load();
			container.addMediaElement( bug );
			
			//Adds the container to the stage
			this.addChild( container );
		}
		
		
		/**
		 * Method to layout control bar. Source is a SWC in the /lib folder
		 * Instances are basic MovieClip symbols set to export for ActionScript
		 */
		protected function initControlBar():void
		{
			controlBar = new Sprite();
			
			controlBG = new ControlBG();
			
			pauseButton = new PauseButton();
			pauseButton.y = 2;
			pauseButton.useHandCursor = true;
			pauseButton.buttonMode = true;
			pauseButton.addEventListener( MouseEvent.CLICK, _onPauseClick );
			
			playButton = new PlayButton();
			playButton.y = 2;
			playButton.x = 43;
			playButton.useHandCursor = true;
			playButton.buttonMode = true;
			playButton.addEventListener( MouseEvent.CLICK, _onPlayClick );
			
			progressBG = new ProgressBG();
			progressBG.y = 2;
			progressBG.x = 87;
			progressBG.useHandCursor = true;
			progressBG.buttonMode = true;
			progressBG.addEventListener( MouseEvent.CLICK, _onSeek );
			
			progressCurrent = new ProgressCurrent();
			progressCurrent.y = 2;
			progressCurrent.x = 87;
			progressCurrent.mouseEnabled = false;
			
			this.addChild( controlBar );
			controlBar.addChild( controlBG );
			controlBar.addChild( pauseButton );
			controlBar.addChild( playButton );
			controlBar.addChild( progressBG );
			controlBar.addChild( progressCurrent );
		}
		
		/**
		 * Called whenever the media player dispatches that the current media resource has changed its size
		 * 
		 * @param p_evt	Contains the newHeight property used to reposition the controlbar
		 * 
		 */
		protected function _onSizeChange( p_evt:DisplayObjectEvent ):void
		{
			//Marker 6: Reposition appropriately when the media size changes	
			controlBar.y = p_evt.newHeight;
		}
		
		/**
		 * Called whenever the media player dispatches that the current media resource has changed its currentTime
		 * Determines the percent played and sets the current progress bar scalex to match
		 * 
		 * @param p_evt	Contains the time property used to determine the current time
		 * 
		 */
		protected function _onProgress( p_evt:TimeEvent ):void
		{	
			//Marker 7: Set the scalex of the current progress bar to the current time divided by the duration of the media	
			progressCurrent.scaleX = p_evt.time / player.duration;
		}
		
		
		
		//USER EVENT HANDLERS
		protected function _onPauseClick( p_evt:MouseEvent ):void
		{
			//Marker 3: Add call to pause method
			player.pause();
		}
		
		
		protected function _onPlayClick( p_evt:MouseEvent ):void
		{
			//Marker 4: Add call to pause method
			player.play();
		}
		
		/**
		 * Tell the media player to seek to time determined by dividing the mouse x-position 
		 * of the seek click by the total width of the progress track multiplied by the total duration
		 * 
		 * @param p_evt
		 * 
		 */
		protected function _onSeek( p_evt:MouseEvent ):void
		{
			//Marker 8: Call the seek method of the MediaPlayer passing it the appropriate time determined by the click on the progress track
			var seekTo:Number = player.duration * ( p_evt.target.mouseX/p_evt.target.width );
			player.seek( seekTo );
		}

		private function onPluginLoad(event:MediaFactoryEvent):void 
		{ 
			trace("The plug-in loaded successfully."); 
		}
		
		private function onPluginLoadError(event:MediaFactoryEvent):void 
		{ 
			trace("The plug-in failed to load: "+event.toString());
		} 
	}
}